      program ambisolv
      implicit double precision (a-h,o-z)
      character*4 sta,staR      

c   ---------------------------------------------------------------
c
c
c   [sta,staR,isatR,isat,sec,DDLc,DDLw,DDLi,DDSTEC,DDBc] --->
c
c                                      |ambisolv| ---> [N1,N2,Nw,Lc]  
c
c
c   ==> [sta,staR,isatR,isat,iarc,sec,DDxNw,DDNw,DDxN1,DDN1,DDN2,
c         1   2    3     4     5   6    7    8     9 a  10   11  
c
c                        DDLi,DDLi_r,DDSTEC,DDBc,DDBc_r,DDLc,DDLc_r] 
c                         12   13      14    15    16    17   18    
c
c  ..................................................................
c
c     where:
c     -----
c
c      "DD" indicates double differences.
c
c      DDNw:   exact value (integer) of the ambiguity wide-lane 
c      DDxNw:  approximate value (before rounding up) of the
c                                             ambiguity wide-lane
c
c      DDN1:   exact value (integer) of the ambiguity in L1 
c      DDxN1:  approximate value (before rounding up) of the 
c                                                 ambiguity in L1
c
c      DDN2 :  exact value (integer) of the ambiguity in L2
c      
c      DDLi  : Ionospheric combination before repairing (fixing 
c                                                   ambiguities)
c
c      DDLi_r: Repaired ionospheric combination (i.e., with 
c                      ambiguities fixed to their exact values)
c      DDSTEC: Prediction of STEC used to repair the ambiguities
c
c      DDBc  : Bias in ionosphere-free combination before repairing
c
c      DDBc_r: Bias in repaired ionosphere-free combination
c
c      DDLc  : Ionosphere-free combination before repairing
c      DDLc_r: Repaired ionosphere-free combination
c
c     
c     Note:
c        DDLi,DDSTEC are expressed in m_LI
c
c        Bc is the bias of the phase Lc estimated by the filter
c        Lc=Lc_inambigua+Bc
c
c
c    Execute:
c    --------
c            cat DDbell_ebre_21.dat |ambisolv > DDbell_ebre_21.amb
c
c
c
c      @gAGE (Research group of Astronomy and GEomatics). UPC
c   -----------------------------------------------------------------


c      ----Constants ------------------
        f1=1.54d0*1.023d9
        f2=1.20d0*1.023d9
        c=299792458.d0
        xlambda1=c/f1
        xlambda2=c/f2
        xlambdaC=c/(f1+f2)
        xlambdaW=c/(f1-f2)
        gamma=(77.d0/60.d0)**2
        fact=(77.d0/60.d0)/((77.d0/60.d0)**2-1.d0)
c       --------------------------------
                                                        
	

10      continue
        read(*,*,end=100) sta,staR,isatR,isat,iarc,sec,
     *                    DDLc,DDLw,DDLi,DDSTEC,DDBc

        xNw=(DDLw-DDLc-DDSTEC*fact+DDBc)/xlambdaW
        Nw=nint(xNw)
	
        xN1=(DDLi-DDSTEC-Nw*xlambda2)/(xlambda1-xlambda2)
        N1=nint(xN1)
c        print *,"AA",isat,iarc,sec,xNw,Nw,xN1,N1,DDBc


c       Reconstruction of the Li_inambigua (DDLi_r): .........
        DDLi_r=DDLi-dble(N1)*(xlambda1-xlambda2)-dble(Nw)*xlambda2
        err_DDLi=DDLi_r-DDSTEC
c       ..................................................... 

        N2=N1-Nw

        DDBc_r=(gamma*xlambda1*dble(N1)-xlambda2*dble(N2))/
     *       (gamma-1.d0)


c       Reconstruction of the Lc_inambigua (DDLc_r): ........          
        DDLc_r=DDLc-DDBc_r
c       ....................................................


        write(*,'(2(a4,1x),3(i2,1x),1x,f8.2,1x,f9.4,1x,i5,1x,
     *   f9.4,2(1x,i5),3(1x,f6.4),2x,2(1x,f9.4),2(1x,f14.4))') 
     *   sta,staR,isatR,isat,iarc,sec,xNw,Nw,xN1,N1,N2,DDLi,
     *   DDLi_r,DDSTEC,DDBc,DDBc_r,DDLc,DDLc_r


        goto 10
100     continue        

        end

c       f77 -o ambisolv ambisolv.f
