c234567
        program coord_ems_P
        implicit double precision (a-h,o-z)
        dimension a(38)

c  ----------------------------------------------------------------------
c       Applying the PSEUDORANGE algorithm, this program computes the
c       satellite coordinates (WGS-84) at the EMISSION time, in the ECEF
c       system at the RECEPTION time. It provides also the difference
c       between the reception time (in the receiver clock) and the
c       emission time (in GPS time).
c
c
c       INPUT:
c       -----
c        - Siganl reception time (in the receiver clock) 
c            iyear: year
c            idoy:  Day-Of-year
c            sec:   Seconds of day 
c            P:     Pseudorange measurement 
c            (x_sta,y_sta,z_sta): receiver coordinates
c                                 WGS'84 (in meters)
c                             
c        - Navigation message (broadcast data)
c          (in RINEX format)
c
c            [sat, year,mon,day,h,m,sec, a0,a1,a2     ]
c            [                    IODE,Crs,dn,xMo,    ]
c            [                    Cuc, e, Cus,a12,    ]
c            [                    toe,Cic,Omgg,Cis,   ]
c            [                    xIo,Crc,omgp,Omgd   ]
c            [                    xIDOT,xx,GPS_Week,xx]
c            [                    SVac,SVh,TGD,IODC   ]
c            [                    xx, xx , xx ,xx     ]
c
c      OUTPUT:
c      ------
c          Satellite coordinates (WGS84) at the EMISSION time
c          in the ECEF system tied to the Earth at the RECEPTION time.
c
c     dt: t_recepcion[reloj_receptor] - t_emision[t_GPS] (segundos)
c
c  --------------------------------------------------------------------
c
c Execution example:
c
c a) Make the file "data_ems_P" with the following data (all data in a 
c    single line)
c
c 1998 286 38230 23585247.703 4789031  176612 4195008
c 14 98 10 13 12 0 0 +5.65452501178E-06 +9.09494701773E-13 +0.000000000000E+00
c +1.28000000000E+02 -6.10000000000E+01 +4.38125402624E-09 +8.198042513605E-01
c -3.31364572048E-06 +1.09227513894E-03 +5.67547976971E-06 +5.153795101166E+03
c +2.16000000000E+05 -6.33299350738E-08 +1.00409621952E+00 -3.725290298462E-09
c +9.73658001335E-01 +2.74031250000E+02 +2.66122811383E+00 -8.081050495434E-09
c -1.45720352451E-10 +1.00000000000E+00 +9.79000000000E+02 +0.000000000000E+00
c +3.20000000000E+01 +0.00000000000E+00 -2.32830643654E-09 +1.280000000000E+02
c +2.08818000000E+05 +0.00000000000E+00 +0.00000000000E+00 +0.000000000000E+00
c
c
c b) Execute:
c
c       cat data_ems_P | coord_ems_P
c
c
c
c ----------------------------------------------------------------------------
                                                                                                        


c       Parameters Declaration : ...........
c       - Light speed (m/s)
           c=299792458.d0
c       - WGS-84 Earth rotation rate (rad/s)
          om_e=7.2921151467d-5
c       ....................................


      read(*,*,end=10) iyear,idoy,sec,P,x_sta,y_sta,z_sta,a 
10    continue  



c   ==> GPS satellite clock Offset with GPS time:

        toc=a(5)*3600.d0+a(6)*60.d0+a(7)
        dt_sat=a(8)+a(9)*(sec-toc)+a(10)*(sec-toc)**2
        

c   ==> EMISION time  (in GPS time)<==
        sec1=sec-P/c-dt_sat


c   ==> Satellite coordinates at the EMISION time (in GPS time)
c       [in the ECEF reference system tied to the Earth at the
c        EMISION time]
        call orbit(iyear,idoy,sec1,a,x0,y0,z0,Ek)


c   ==> Satellite coordinates at the EMISION time (in GPS time)
c       [in the ECEF reference system tied to the Earth at the 
c        RECEPTION time]

c       ... Geometric Range (in seconds)
        dt=dsqrt((x0-x_sta)**2+(y0-y_sta)**2+(z0-z_sta)**2)/c

c       Coordinate transformation to the ECEF system tied to 
c       the Earth at the RECEPTION time.
c       .... Earth Rotation during the time "dt" 
        x=x0+y0*om_e*dt
        y=y0-x0*om_e*dt
        z=z0
c       ...................................................                

        write(*,'(f4.1,1x,f14.4,1x,f14.4,1x,f14.4,f14.4,2(1x,f9.3))'),
     *  a(1),sec1,x,y,z,sec1-sec,Ek

        end


c      f77 -o  coord_ems_P coord_ems_P.f  orbit.o 
