cx234567
      implicit double precision (a-h,o-z)
       dimension p(6),r0(3),r(3),v(3),rot1(9),rot2(9),
     * rot3(9), rot(9)

c   ..........................................................
c
c
c     [a,e,i,OMEGA,omega,M] ---> |ele_orb2rv| ----> [r,v]
c                                              (equatorial)
c                                              x => Aries
c                                              z => North Pole
c     (units: km and rad)              (units: km and km/s)
c
c     NOTE:  TERRESTRIAL coordinates!!: ---------------------
c   If W (asc. node long.) referred to Aries  => x to Aries
c   If W (asc. node long.) referred to Green. => x to Green.
c     WARNING: in this last case the velocity would correspond to
c              an inertial system rotating an angle=t_sider.
c              about z axis.
c
c       Note: in the files *eph and *b
c              W is referring to Greenwich
c
c   ----------------------------------------------------------
c     Note: M=n(t-T)  with n=dsqrt(gm/a**3)
c   ..........................................................
c
c   Example1:
c   echo "26549. 0.014 0.946 -1.246 -2.476 0.781" |ele_orb2rv
c
c   Result:
c        -15366.34 -2287.86 -21205.69 .99 -3.77 -.36
c
c
c      @gAGE (Research group of Astronomy and GEomatics). UPC
c ------------------------------------------------------------

        gm=398600.5d0
        c=299792458.d0

        pi=3.1415926535898d0
        om_e=7.2921151467d-5

10      continue
        read (*,*,end=100) p
        call kepler(p(6),p(2),ex)
        rr=p(1)*(1.d0-p(2)*dcos(ex))
        xv=datan2(dsqrt(1.d0-p(2)**2)*dsin(ex),dcos(ex)-p(2))
        call rotate(3,-p(5),rot3)
        call rotate(1,-p(3),rot1)
        call prod(rot1,3,3,rot3,3,3,rot2)
        call rotate(3,-p(4),rot3)
        call prod(rot3,3,3,rot2,3,3,rot)

c       calculo de la posicion r:
        r0(1)=rr*dcos(xv)
        r0(2)=rr*dsin(xv)
        r0(3)=0.d0
        call prod(rot,3,3,r0,3,1,r)

c       calculo de la velocidad v:
        xna=dsqrt(gm/p(1))
        b=p(1)*dsqrt(1.d0-p(2)**2)
        v(1)=xna/rr*(rot(4)*b*dcos(ex)-rot(1)*p(1)*dsin(ex))
        v(2)=xna/rr*(rot(5)*b*dcos(ex)-rot(2)*p(1)*dsin(ex))
        v(3)=xna/rr*(rot(6)*b*dcos(ex)-rot(3)*p(1)*dsin(ex))
        write(*,*) r,v
        goto 10

100     end
c ------------------------------------------------------------
       subroutine kepler(xm,e,ex)
       implicit double precision (a-h,o-z)
       eps=1.d-12
       ex=xm
10     dex=xm-(ex-e*dsin(ex))
       ex=ex+dex
       if (dex.gt. eps) goto 10
       return
       end
c ------------------------------------------------------------
       subroutine rotate(iaxis,angle,rot)
       implicit DOUBLE PRECISION (A-H,O-Z)
       dimension rot(9)

c -----------------------------------------------------------
c      It provides the (Fortran vectored) rotation matrix "rot"
c      for a given angle "angle" arround the coordinate axis
c      [1<=>x], [2<=>y],[3<=>z].
c
c      @gAGE (Research group of Astronomy and GEomatics). UPC
c ------------------------------------------------------------

       do i=1,9
           rot(i)=0.d0
       enddo

       if (iaxis .eq. 1) then
           rot(1)=1.d0
           rot(5)=dcos(angle)
           rot(6)=-dsin(angle)
           rot(8)=dsin(angle)
           rot(9)=dcos(angle)
         elseif (iaxis .eq. 2) then
           rot(5)=1.d0
           rot(1)=dcos(angle)
           rot(7)=-dsin(angle)
           rot(3)=dsin(angle)
           rot(9)=dcos(angle)
         elseif (iaxis .eq. 3) then
           rot(9)=1.d0
           rot(1)=dcos(angle)
           rot(2)=-dsin(angle)
           rot(4)=dsin(angle)
           rot(5)=dcos(angle)
       endif

       return
       end
c ------------------------------------------------------------
       subroutine prod(A,m,l,B,ll,n,C)
       implicit DOUBLE PRECISION (A-H,O-Z)
       dimension A(*),B(*),C(*)

c ------------------------------------------------------------
c      It computes the product of a mxl matrix A and a llxn
c      matrix B. The result is the mxn matrix C. The matrix
c      must be general matrix, Fortran vectored (by collumns).
c
c
c      @gAGE (Research group of Astronomy and GEomatics). UPC
c ------------------------------------------------------------


       do i=1,m*n
          C(i)=0.d0
       enddo

       if (l.ne.ll) then
          print *, "ERROR: dimension matrix"
        goto 100
       endif

        do 10 i=1,m
          do 20 j=1,n
            C(i+m*(j-1))=0.d0
            do 30 k=1,l
              C(i+m*(j-1))=C(i+m*(j-1))+A(i+m*(k-1))*B(k+l*(j-1))
30          enddo
20        enddo
10      enddo

100    continue
       return
       end
c ------------------------------------------------------------


c      f77 -o  ele_orb2rv  ele_orb2rv.f
