c234567
         program eph2txt
         implicit double precision (a-h,o-z)
         character line*80,dati*80,datb*80,datc*80
         dimension ai(61)

c --------------------------------------------------------
c  This programs reads the RINEX ephemeris file and writes
c  the orbital elements in a collumn format (one row for
c  each satellite).
c
c  @gAGE (Research group of Astronomy and GEomatics). UPC
c --------------------------------------------------------

          call getarg(1,dati)
          open (10,file=dati)

          write(datb,'(a,a)') dati(1:7),".b"
          write(datc,'(a,a)') dati(1:7),".clocks"
          open (11,file=datb)
          open (12,file=datc)

          w_earth=7.2921151467d-5
          iflag=0

10        continue
          line=" "
          do i=1,61
           ai(i)=0.d0
          enddo

          read (10,'(a)',end=100) line

          if (line(61:73).eq."END OF HEADER") iflag=1
          if (iflag.eq.0) goto 10

c         Reading ephemeris ..........................
          if (line(1:2).ne." ") then
             read(line,*,err=10) (ai(i),i=1,6)

             if (ai(2).lt.1000.d0) then
              if (ai(2).lt.80.d0) then
                ai(2)=ai(2)+2000.d0
               else
                ai(2)=ai(2)+1900.d0
              endif
             endif

             read(line(19:22),*) ai(7)

             do k=23,61,19
               i=6+(k+15)/19
               read(line(k:k+18),*) ai(i)
             enddo      

             do i=1,7
               read (10,'(a)',end=100) line
               do k=4,61,19
                 j=4*i+(k+15)/19+6
                 read(line(k:k+18),*) ai(j)
               enddo
             enddo
 
            iPRN=nint(ai(1))
            a=ai(18)*ai(18)
            e=ai(16)
            xi=ai(23)
            xlambda=ai(21)-w_earth*ai(19)
            omega=ai(25)
            xMo=ai(14)        
        
            sec=(ai(5)*60.d0+ai(6))*60.d0+ai(7)
            call sub_cal2dgpsws(ai(2),ai(3),ai(4),sec,idoy,nw,sw)

            write(11,70) iPRN,idoy,sec,a,e,xi,xlambda,omega,xMo
70          format(i2.2,1x,i3,1x,f8.2,1x,f16.7,5(1x,f18.12))

            write(12,80) iPRN,idoy,sec,(ai(k),k=8,10)
80          format(i2.2,1x,i3,1x,f8.2,3(1x,e20.12))

         endif
c        ...............................................

         goto 10
100      continue
         end

c  ------------------------------------------------------------------------

         subroutine sub_cal2dgpsws(year,xmonth,day,sec,idoy,nw,sw)
         implicit double precision (a-h,o-z)
 
c        --------------------------------------------------------
c        Compute the  DOY from YEAR MONTH DAY
c
c        1) Until February 28th of 2100 it calculate consitent doys
c           days. Outside this range there will be a discrepancy which
c           increases by one day for every non-leap century year.
c        2) If the year is given with only two digits, the range is
c           between 1980 to 2079
c        --------------------------------------------------------
 
c         print *, "YYY",year,xmonth,day,sec
         xy=year
c        two digits iyear control ...........
         if (xy.lt.100.d0) then
            if (xy.lt.80.d0) then
               xy=xy+2000.d0
             else
               xy=xy+1900.d0
            endif
         endif
c        ....................................
         xm=xmonth
 
         jd0=int(365.25d0*(xy-1.d0))+428
         if (xm.le.2.d0) then
           xy=xy-1.d0
           xm=xm+12.d0
         endif
         jd=int(365.25d0*xy)+int(30.6001d0*(xm+1.d0))-jd0
         doy=dble(jd)+day
         idoy=nint(doy)
 
c        GPS day: (1980jan6.0 => JD=2444244.5 => id_GPS=1.0 )
         id_GPS=jd0+idoy-723263
c        Day of week:
         idw=mod(id_GPS,7)
c        Number of GPS week:
         nw=(id_GPS-idw)/7
c        seconds into the week:
         sw=dble(idw)*86400.d0+sec
 
         return
         end
c        -----------------------------------------------------------                                                         

c     f77 -o eph2txt eph2txt.f
