c234567890
           program dry_nomG
           implicit double precision (a-h,o-z)
           character  sta*4    

	
c       -----------------------------------------------------------
c       Model of Elgered et al (1991) to calculate the 
c       NOMINAL HYDROSTATIC delay
c
c
c         sta x y z ---> |dry_nomG| ---> NOMINAL_HYDROSTATIC_delay
c             CTS(m)                           (m of delay)
c
c
c       echo "madr 4849202.52 -360329.17 4114913.06" |dry_nomG 
c
c             ==> madr  4849202.52 -360329.17  4114913.06  2.08856326 
c                                                          ^^^^^^^^^^
c                                                         (m of delay)   
c
c      @gAGE (Research group of Astronomy and GEomatics). UPC
c      -----------------------------------------------------------

10      continue
        read(*,*,end=100) sta,x,y,z      

c       from cartesian to geodetic coordinates............
	call sub_cart2geod(x,y,z,fi,xlambda,ht)      
c       ..................................................


c       Hidrostatic delay .......................................
c       ...(following GIPSY /goa/file_formats/qregres.nml)... 
	DRY = 1.013d0*2.27*exp(-0.116d-03*ht)
c       .........................................................

	print *, sta,x,y,z,dry

	goto 10
100     continue
	end


c -------------------------------------------------------------------------


        subroutine sub_cart2geod(x,y,z,fi,xl,h)  

	        implicit double precision (a-h,o-z)

c       -----------------------------------------------
c       Coordinates conversion from cartesian (x,y,z)
c       to geodetic (fi, lambda, h) WGS84
c
c           x y z ---> |cart2geod| ---> fi, lambda, h
c
c
c       Execute, for instance:
c
c       echo 4210520.62 1128205.6 4643227.5 | cart2geod
c
c       ==> fi, lambda, h  47.  15.  2000.00277
c
c       -----------------------------------------------
c
c
c
c       ....values declaration........................
        tol=1.d-9
c         WGS84 parameters (in meters)
          a=6378137.d0
          b=6356752.314d0
          e2=(a**2-b**2)/a**2
c       ..............................................


        pi=3.1415926535898d0

        xl=datan2(y,x)

        p=dsqrt(x**2+y**2)
        fi=datan(z/p/dsqrt(1.d0-e2))
        fia=fi
20      continue
        xn=a**2/dsqrt((a*dcos(fi))**2+(b*dsin(fi))**2)
        h=p/dcos(fi)-xn
        fi=datan(z/p/(1.d0-e2*xn/(xn+h)))
        if(dabs(fi-fia).gt.tol) then
           fia=fi
           goto 20
         else
c           print *, "fi, lambda, h",fi*180.d0/pi,xl*180.d0/pi,h
        endif
100     continue


c       xn=a**2/dsqrt((a*dcos(fi))**2+(b*dsin(fi))**2)
c       x=(xn+h)*dcos(fi)*dcos(xl)
c       y=(xn+h)*dcos(fi)*dsin(xl)
c       z=(b**2/a**2*xn+h)*dsin(fi)
c       print *, x,y,z

	return
        end                                              
c --------------------------------------------------------------------------

                                                                                 
c       f77 -o  dry_nomG  dry_nomG.f
