c1234567890
 
        program trop_niell
        implicit double precision (a-h,o-z)
        dimension avr(3,5),amp(3,5),awt(3,5)
 
c  ----------------------------------------------------------------
c  Implementation of the NIELL mapping function for the troposphere
c  Niell (1996), Journal of Geophysical Research, 101, 3227-3247
c
c   [doy,xlat,H],elev --->  | trop_niell | ---> map_HIDR map_WET
c     ^   ^   ^   ^
c     |   |   |   |__elevation angle (in rad)
c     |   |   |                       ^^^^^^
c     |   |   |__ site heigh above sea level (in meters)
c     |   |
c     |   |__ site latitude (in rad)
c     |                     ^^^^^^^^
c     |__Time from January 0.0 (UT in days)
c
c
c   Example:
c          echo 113 32.3704 -11.608262 43.80250454 | trop_niell
c
c               ===>    xm_hydr,xm_wet  1.4427585  1.44383147
c
c
c
c      @gAGE (Research group of Astronomy and GEomatics). UPC
c  ----------------------------------------------------------------
 
 
        read(*,*) doy,xlat,H,elv
 
        pi=3.1415926535898d0

        elv=elv*pi/180.d0
 
        xlat=dabs(xlatt)*180.d0/pi
 
c   ---Coefficients of Hydrstatic Mapping Function (nmfh2.0)---
 
c       ........................Average.......................
        data (avr(1,j),j=1,5) /
     :  1.2769934d-3, 1.2683230d-3, 1.2465397d-3, 1.2196049d-3,
     :  1.2045996d-3 /
        data (avr(2,j),j=1,5) /
     :  2.9153695d-3, 2.9152299d-3, 2.9288445d-3, 2.9022565d-3,
     :  2.9024912d-3 /
        data (avr(3,j),j=1,5) /
     :  62.610505d-3, 62.837393d-3, 63.721774d-3, 63.824265d-3,
     :  64.258455d-3 /
c       .......................................................
 
c       .......................Amplitude.......................
        data (amp(1,j),j=1,5) /
     :  0.d0,         1.2709626d-5, 2.6523662d-5, 3.4000452d-5,
     :  4.1202191d-5 /
        data (amp(2,j),j=1,5) /
     :  0.d0,         2.1414979d-5, 3.0160779d-5, 7.2562722d-5,
     :  11.723375d-5 /
        data (amp(3,j),j=1,5) /
     :  0.d0,         9.0128400d-5, 4.3497037d-5, 84.795348d-5,
     :  170.37206d-5 /
c       .......................................................
 
c       ....................Height Correction .................
        data aht / 2.53d-5 /
        data bht / 5.49d-3 /
        data cht / 1.14d-3 /
c       .......................................................
c   -------------------------------------------------------------
 
c   ---Coefficients of Wet Mapping Function (nmfw2.0)------------
        data (awt(1,j),j=1,5) /
     :  5.8021897d-4, 5.6794847d-4, 5.8118019d-4, 5.9727542d-4,
     :  6.1641693d-4 /
        data (awt(2,j),j=1,5) /
     :  1.4275268d-3, 1.5138625d-3, 1.4572752d-3, 1.5007428d-3,
     :  1.7599082d-3 /
        data (awt(3,j),j=1,5) /
     :  4.3472961d-2, 4.6729510d-2, 4.3908931d-2, 4.4626982d-2,
     :  5.4736038d-2 /
 
c   -------------------------------------------------------------
 
 
c   ---MAPPING FUNCTIONS-----------------------------------------
 
c       ...Hidrostatic mapping ..................................
c       +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c       WARNING:[There is an mistake in the paper ("+==>-")]
c       +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        a=xinterp(xlat,avr,1)-
     *    xinterp(xlat,amp,1)*dcos(2.d0*pi*(doy-28.d0)/365.25d0)
 
        b=xinterp(xlat,avr,2)-
     *    xinterp(xlat,amp,2)*dcos(2.d0*pi*(doy-28.d0)/365.25d0)
 
        c=xinterp(xlat,avr,3)-
     *    xinterp(xlat,amp,3)*dcos(2.d0*pi*(doy-28.d0)/365.25d0)
 
        xm_hydr=xm(elv,a,b,c)+
     *     (1.d0/dsin(elv)-xm(elv,aht,bht,cht))*H/1000.d0
 
c       ........................................................
 
c       ...Wet mapping .........................................
        a=xinterp(xlat,awt,1)
        b=xinterp(xlat,awt,2)
        c=xinterp(xlat,awt,3)
c       print *, a,b,c
 
        xm_wet=xm(elv,a,b,c)
c       ........................................................
 
        print *, "xm_hydr,xm_wet", xm_hydr,xm_wet
c   --------------------------------------------------------------
        end
 
                                                   
 
c   --------------------------------------------------------------
        double precision function xm(elv,a,b,c)
        implicit none
        double precision elv,a,b,c,xn,xd
c       +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c       WARNING: there is an error in the mapping formula of the paper
c       +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        xn=1.d0+a/(1.d0+b/(1.d0+c))
        xd=dsin(elv)+a/(dsin(elv)+b/(dsin(elv)+c))
 
c       print *, "xn,xd,elv", xn,xd,elv
c        print *, xn/xd,1.d0/dsin(elv),dsin(elv)
        xm=xn/xd
 
        end
c   --------------------------------------------------------------
 
 
        double precision function xinterp(x,v,i)
        implicit none
        double precision x,x1,y1,x2,y2,v(3,5)
        integer i,j
 
c        print *,x
 
        if (x.le.15.d0) then
           xinterp=v(i,1)
           goto 100
        endif
        if (x.ge.75.d0) then
           xinterp=v(i,5)
           goto 100
        endif
 
        j=1+int(x/15.d0-1)
 
        x1=15.d0*dble(j)
        y1=v(i,j)
        x2=15.d0*dble(j+1)
        y2=v(i,j+1)
 
        xinterp=(x-x2)/(x1-x2)*y1+(x-x1)/(x2-x1)*y2
c        print *,x1,x2,y1,y2
100     continue
 
        end
c   --------------------------------------------------------------
                                                                                                                                                
 
c     f77 -o trop_niell trop_niell.f  
