        subroutine klob(t,x_sta,y_sta,z_sta,
     *                    x_sat,y_sat,z_sat,
     *                    alpha0,alpha1,alpha2,alpha3,
     *                    beta0,beta1,beta2,beta3,Tiono)       

        implicit double precision (a-h,o-z)

c ------------------------------------------------------------
c  Klobuchar model implementation for the calculation of the
c  ionospheric delay.
c
c  INPUT:
c        t   : observation epoch (seconds in the day)
c        (x_sta,y_sta,z_sta): receiver coordinates (WGS84) meters 
c        (x_sat,y_sat,z_sat): satellite coordinates (WGS84) meters 
c
c        KLobuchar coefficients (file klobuchar.dat):
c           alpha0,alpha1,alpha
c           beta0,beta1,beta2,beta3
c  OUTPUT:
c        Tiono: ionospheric delay (meters L1)
c
c
c      @gAGE (Research group of Astronomy and GEomatics). UPC    
c ------------------------------------------------------------

c   ... parameter declarations ......
       	c=299792458.d0   
       	pi=3.1415926535898d0       
c   .................................


c  ============================================================
c  Calculate the geodetic user longitude and latitude
        call car2geo(x_sta,y_sta,z_sta,ulon,ulat,h)  

c  Calculate the elevation and aziuth of receiver-satelite ray
        x1=x_sat-x_sta
        y1=y_sat-y_sta
        z1=z_sat-z_sta
        rho=dsqrt(x1**2+y1**2+z1**2)         
        slat=dsin(ulat)
        slon=dsin(ulon)
        clat=dcos(ulat)
        clon=dcos(ulon)         
        G1=(-slat*clon*x1-slat*slon*y1+clat*z1)/rho
        G2=(-slon*x1+clon*y1)/rho
        G3=( clat*clon*x1+clat*slon*y1+slat*z1)/rho          
        elev=dasin(G3)
        azim=datan2(G2,G1)        
c  ============================================================

c   Calculate the Earth-centered angle fm (in semicircles): ........
    	fm=0.0137d0/(elev/pi+0.11d0)-0.022d0

c   Compute the subionospheric latitude, xilat (in semicircles): ...
    	xilat=ulat/pi+fm*dcos(azim)
    	if (xilat.gt. 0.416d0) xilat= 0.416d0
    	if (xilat.lt.-0.416d0) xilat=-0.416d0       

c   Compute the subionospheric longitude, xilon (in semicircles): .. 
    	xilon=ulon/pi+fm*dsin(azim)/dcos(xilat*pi)
 
c   Find the Geomagnetic latitude gmlat (in semicircles): ...
    	gmlat=xilat+0.064d0*dcos((xilon-1.617d0)*pi)

c   Find the local time at the subionspheric point: tsub (in sec.)
    	tsub=4.32d4*xilon+t
    	if (tsub .gt. 86400.d0) tsub=tsub-86400.d0
    	if (tsub .lt. 0.d0) tsub=tsub+86400.d0   

c   Convert to slant time delay (compute the slant factor F):
    	F=1.d0+16.d0 *(0.53d0-elev/pi)**3

c  Compute the ionospheric time delay sTEC (in meters of delay):
   	alpha=alpha0+alpha1*gmlat+alpha2*gmlat**2+alpha3*gmlat**3  
   	beta= beta0+beta1*gmlat+beta2*gmlat**2+beta3*gmlat**3
        if (alpha.lt.0.d0) alpha=0.d0
        if (beta.lt.72000.d0) beta=72000.d0
   	x=2.d0*pi*(tsub-50400.d0)/beta

   	if (dabs(x).le.pi/2.d0) then
     	   y=alpha*(1.d0-x**2/2.d0+x**4/24.d0)
    	 else
           y=0.d0
   	endif

   	Tiono=F*(5.d-9+y)*c

        return
        end


c       f77 -c klob.f
